import { HttpStatusCode } from "@rocket.chat/apps-engine/definition/accessors";
import { IApiResponse } from "@rocket.chat/apps-engine/definition/api";
import { SCIMUser } from "../scim/User";
import { Context } from "./Context";
import { IScimEndpoint, ScimEndpoint } from "./ScimEndpoint";

export class UserEndpoint extends ScimEndpoint implements IScimEndpoint {
    public path = "Users/:id";

    public async _get(ctx: Context): Promise<IApiResponse> {
        const o = await ctx.rc.user.info(ctx.id());
        this.handleError(o);
        const user = SCIMUser.fromRC(o.user);
        const m = await ctx.store.getUser(ctx.id());
        if (m) {
            user.externalId = m.externalId;
        }
        return this.success(user);
    }

    public async _put(ctx: Context): Promise<IApiResponse> {
        const u = SCIMUser.fromPlain(ctx.content());
        const o = await ctx.rc.user.update({
            userId: ctx.id(),
            data: {
                email: u.getEmail(),
                name: u.displayName || undefined,
                username: u.userName,
                active: u.active,
                verified: true,
                customFields: {
                    scimExternalId: u.externalId,
                },
            },
        });
        this.handleError(o);
        await ctx.store.saveUser(ctx.id(), { externalId: u.externalId });
        const user = SCIMUser.fromRC(o.user);
        user.externalId = u.externalId;
        return this.success(user);
    }

    public async _delete(ctx: Context): Promise<IApiResponse> {
        const o = await ctx.rc.user.delete({
            userId: ctx.id(),
            confirmRelinquish: true,
        });
        this.handleError(o);
        await ctx.store.deleteUser(ctx.id());
        return this.response({
            status: HttpStatusCode.NO_CONTENT,
        });
    }
}
